@@ -1,6 +1,6 @@
 // W32DirectSoundPlayer.cc --- Sound player
 //
-// Copyright (C) 2002 - 2010 Raymond Penners & Ray Satiro
+// Copyright (C) 2002 - 2010, 2012, 2013 Raymond Penners & Ray Satiro
 // All rights reserved.
 //
 // This program is free software: you can redistribute it and/or modify
@@ -31,7 +31,7 @@
 #include <process.h>
 #include <mmsystem.h>
 #include <mmreg.h>
-#include <dxerr.h>
+#include <dxerr8.h>
 #ifndef DXGetErrorString8
 #define DXGetErrorString8 DXGetErrorString
 #endif
@@ -111,58 +111,19 @@ registry_set_value(const char *path, const char *name,
 //! Constructor
 W32DirectSoundPlayer::W32DirectSoundPlayer()
 {
-  direct_sound = NULL;
 }
 
 
 //! Destructor
 W32DirectSoundPlayer::~W32DirectSoundPlayer()
 {
-  if (direct_sound != NULL)
-    {
-      direct_sound->Release();
-      direct_sound = NULL;
-    }
 }
 
 //!
 void
 W32DirectSoundPlayer::init(ISoundDriverEvents *events)
 {
-  HRESULT             hr = S_OK;
-
   this->events = events;
-  
-  if (direct_sound != NULL)
-    {
-      direct_sound->Release();
-      direct_sound = NULL;
-    }
-
-  try
-    {
-      hr = DirectSoundCreate8(NULL, &direct_sound, NULL);
-      if (FAILED(hr) || direct_sound == NULL)
-        {
-          throw Exception(string("DirectSoundCreate8") + DXGetErrorString8(hr));
-        }
-
-      hr = direct_sound->SetCooperativeLevel(GetDesktopWindow(), DSSCL_PRIORITY);
-      if (FAILED(hr))
-        {
-          throw Exception(string("IDirectSound_SetCooperativeLevel") + DXGetErrorString8(hr));
-        }
-    }
-  catch (Exception)
-    {
-      if (direct_sound != NULL)
-        {
-          direct_sound->Release();
-          direct_sound = NULL;
-        }
-      
-      throw;
-    }
 }
 
 
@@ -198,11 +159,11 @@ W32DirectSoundPlayer::play_sound(string wavfile)
 {
   TRACE_ENTER_MSG( "W32DirectSoundPlayer::play_sound", wavfile);
 
-  if (direct_sound != NULL && wavfile != "")
+  if (wavfile != "")
     {
       DWORD id;
-
-      SoundClip *clip = new SoundClip(direct_sound, wavfile, events);
+      
+      SoundClip *clip = new SoundClip(wavfile, events);
       CloseHandle(CreateThread(NULL, 0, play_thread, clip, 0, &id));
     }
 
@@ -214,7 +175,7 @@ bool
 W32DirectSoundPlayer::get_sound_enabled(SoundEvent snd, bool &enabled)
 {
   char key[MAX_PATH], val[MAX_PATH];
-  
+
   strcpy(key, "AppEvents\\Schemes\\Apps\\Workrave\\");
   strcat(key, SoundPlayer::sound_registry[snd].label);
   strcat(key, "\\.current");
@@ -224,7 +185,7 @@ W32DirectSoundPlayer::get_sound_enabled(SoundEvent snd, bool &enabled)
       enabled = (val[0] != '\0');
       return true;
     }
-  
+
   return false;
 }
 
@@ -235,7 +196,7 @@ W32DirectSoundPlayer::set_sound_enabled(SoundEvent snd, bool enabled)
   if (enabled)
     {
       char key[MAX_PATH], def[MAX_PATH];
-      
+
       strcpy(key, "AppEvents\\Schemes\\Apps\\Workrave\\");
       strcat(key, SoundPlayer::sound_registry[snd].label);
       strcat(key, "\\.default");
@@ -250,11 +211,11 @@ W32DirectSoundPlayer::set_sound_enabled(SoundEvent snd, bool enabled)
   else
     {
       char key[MAX_PATH];
-      
+
       strcpy(key, "AppEvents\\Schemes\\Apps\\Workrave\\");
       strcat(key, SoundPlayer::sound_registry[snd].label);
       strcat(key, "\\.current");
-  
+
       registry_set_value(key, NULL, "");
     }
 }
@@ -268,7 +229,7 @@ W32DirectSoundPlayer::get_sound_wav_file(SoundEvent snd, std::string &wav_file)
   strcpy(key, "AppEvents\\Schemes\\Apps\\Workrave\\");
   strcat(key, SoundPlayer::sound_registry[snd].label);
   strcat(key, "\\.current");
-  
+
   if (registry_get_value(key, NULL, val))
     {
       wav_file = val;
@@ -283,8 +244,8 @@ W32DirectSoundPlayer::get_sound_wav_file(SoundEvent snd, std::string &wav_file)
           wav_file = val;
         }
     }
-  
-  return true;;
+
+  return true;
 }
 
 void
@@ -298,7 +259,7 @@ W32DirectSoundPlayer::set_sound_wav_file(SoundEvent snd, const std::string &wav_
     {
       registry_set_value(key, NULL, SoundPlayer::sound_registry[snd].friendly_name);
     }
-  
+
   strcpy(key, "AppEvents\\Schemes\\Apps\\Workrave\\");
   strcat(key, SoundPlayer::sound_registry[snd].label);
   strcat(key, "\\.default");
@@ -337,21 +298,20 @@ W32DirectSoundPlayer::play_thread(LPVOID lpParam)
   catch(...)
     {
     }
- 
-  delete clip;
 
+  delete clip;
+  
   TRACE_EXIT();
   return (DWORD) 0;
 }
 
 
 
-SoundClip::SoundClip(LPDIRECTSOUND8 direct_sound, const string &filename, ISoundDriverEvents *events)
+SoundClip::SoundClip(const string &filename, ISoundDriverEvents *events)
 {
   TRACE_ENTER("SoundClip::SoundClip");
-  direct_sound->AddRef();
 
-  this->direct_sound = direct_sound;
+  this->direct_sound = NULL;
   this->filename = filename;
   this->events = events;
 
@@ -367,12 +327,6 @@ SoundClip::SoundClip(LPDIRECTSOUND8 direct_sound, const string &filename, ISound
 SoundClip::~SoundClip()
 {
   TRACE_ENTER("SoundClip::~SoundClip");
-  if (direct_sound != NULL)
-    {
-      direct_sound->Release();
-      direct_sound = NULL;
-    }
-
   if (sound_buffer != NULL)
     {
       sound_buffer->Release();
@@ -384,7 +338,13 @@ SoundClip::~SoundClip()
       CloseHandle(stop_event);
       stop_event = NULL;
     }
-  
+
+  if (direct_sound != NULL)
+    {
+      direct_sound->Release();
+      direct_sound = NULL;
+    }
+
   delete wave_file;
   wave_file = NULL;
 
@@ -398,6 +358,18 @@ SoundClip::init()
   HRESULT hr = S_OK;
 
   TRACE_ENTER("SoundClip::init");
+
+  hr = DirectSoundCreate8(NULL, &direct_sound, NULL);
+  if (FAILED(hr) || direct_sound == NULL)
+    {
+      throw Exception(string("DirectSoundCreate8") + DXGetErrorString8(hr));
+    }
+
+  hr = direct_sound->SetCooperativeLevel(GetDesktopWindow(), DSSCL_PRIORITY);
+  if (FAILED(hr))
+    {
+      throw Exception(string("IDirectSound_SetCooperativeLevel") + DXGetErrorString8(hr));
+    }
   
   wave_file = new WaveFile(filename);
   wave_file->init();
@@ -424,7 +396,7 @@ SoundClip::init()
       throw Exception(string("IDirectSoundBuffer_CreateSoundBuffer") + DXGetErrorString8(hr));
     }
 
-  LPDIRECTSOUNDNOTIFY8 notify;
+  LPDIRECTSOUNDNOTIFY notify;
   hr = sound_buffer->QueryInterface(IID_IDirectSoundNotify8, (LPVOID*)&notify);
   if (FAILED(hr) || notify == NULL)
     {
@@ -437,7 +409,7 @@ SoundClip::init()
   DSBPOSITIONNOTIFY pn;
   pn.dwOffset = DSBPN_OFFSETSTOP;
   pn.hEventNotify = stop_event;
-  
+
   hr = notify->SetNotificationPositions(1, &pn);
   if (FAILED(hr))
     {
@@ -460,14 +432,14 @@ SoundClip::fill_buffer()
 {
   TRACE_ENTER("SoundClip::fill_buffer");
 
-  HRESULT hr; 
+  HRESULT hr;
   VOID *locked_sound_buffer = NULL;
   DWORD locked_sound_buffer_size = 0;
 
   restore_buffer();
 
-  hr = sound_buffer->Lock(0, sound_buffer_size, 
-                          &locked_sound_buffer, &locked_sound_buffer_size, 
+  hr = sound_buffer->Lock(0, sound_buffer_size,
+                          &locked_sound_buffer, &locked_sound_buffer_size,
                           NULL, NULL, 0L);
   if (FAILED(hr))
     {
@@ -480,11 +452,11 @@ SoundClip::fill_buffer()
 
   if (locked_sound_buffer_size - bytes_read > 0)
     {
-      FillMemory((BYTE*) locked_sound_buffer + bytes_read, 
-                 locked_sound_buffer_size - bytes_read, 
+      FillMemory((BYTE*) locked_sound_buffer + bytes_read,
+                 locked_sound_buffer_size - bytes_read,
                  (BYTE)(wave_file->get_format()->wBitsPerSample == 8 ? 128 : 0));
     }
-  
+
   sound_buffer->Unlock(locked_sound_buffer, locked_sound_buffer_size, NULL, 0);
   sound_buffer->SetCurrentPosition(0);
   TRACE_EXIT();
@@ -516,7 +488,7 @@ SoundClip::restore_buffer()
 {
   TRACE_ENTER("SoundClip::restore_buffer");
   HRESULT hr = S_OK;
-  do 
+  do
     {
       hr = sound_buffer->Restore();
       if (hr == DSERR_BUFFERLOST)
@@ -544,7 +516,7 @@ SoundClip::play()
     {
       events->eos_event();
     }
-  
+
   TRACE_EXIT();
 }
 
@@ -556,7 +528,7 @@ SoundClip::set_volume(int volume)
   if (sound_buffer != NULL)
     {
       long dsVolume;
-      
+
       if (volume == 0)
         {
           dsVolume = -10000;
@@ -565,7 +537,7 @@ SoundClip::set_volume(int volume)
         {
           dsVolume = 100 * (long) (20 * log10((double) volume / 100.0));
         }
-      
+
       dsVolume = CLAMP(dsVolume, -10000, 0);
 
       sound_buffer->SetVolume(dsVolume);
@@ -620,7 +592,7 @@ WaveFile::init()
       TRACE_RETURN("Exception: mmioDescend1");
       throw Exception("mmioDescend1");
     }
-  
+
   if (parent.ckid != FOURCC_RIFF || parent.fccType != mmioFOURCC('W', 'A', 'V', 'E' ))
     {
       TRACE_RETURN("Exception: no Wave");
@@ -630,7 +602,7 @@ WaveFile::init()
   memset((void *)&child, 0, sizeof(child));
 
   child.ckid = mmioFOURCC('f', 'm', 't', ' ');
-  
+
   res = mmioDescend(mmio, &child, &parent, MMIO_FINDCHUNK);
   if (res != MMSYSERR_NOERROR)
     {
@@ -709,14 +681,14 @@ WaveFile::read(BYTE *buffer, size_t size)
       TRACE_RETURN("Exception: mmioGetInfo");
       throw Exception("mmioGetInfo");
     }
-  
+
   int pos = 0;
   do
     {
       size_t copy = mmioInfo.pchEndRead - mmioInfo.pchNext;
 
-      if (copy > 0) 
-        {	
+      if (copy > 0)
+        {
           if (copy > size - pos)
             {
               copy = size - pos;
@@ -730,7 +702,7 @@ WaveFile::read(BYTE *buffer, size_t size)
 
     } while (pos < (int)size && mmioAdvance(mmio, &mmioInfo, MMIO_READ) == 0);
 
-  mmioSetInfo(mmio, &mmioInfo, 0); 
+  mmioSetInfo(mmio, &mmioInfo, 0);
 
   TRACE_EXIT();
   return pos;
